@@ -15,13 +15,13 @@ mod sse;
 mod special;
 
 pub use self::sse::{f32x4, f64x2};
-pub use self::special::f32x2;
+pub use self::special::{f32x2};
 
 cfg_if! {
     if #[cfg(all(feature = "use_avx", target_feature = "avx"))] {
         mod avx;
         pub use self::avx::*;
-        pub use self::special::f32x6;
+        pub use self::special::{f32x6, f64x3};
     } else {
         pub use self::sse::*;
     }
